#!/usr/bin/env python
"""Top-level module for sed_eval"""

# Import all submodules
from . import sound_event
from . import scene
from . import audio_tag
from . import io
from . import util
from . import metric
from . import test

__version__ = '0.2.1'